enum
{
	FUNC_NULL=0,
	FUNC_OPENEX,
	FUNC_OPENDIR,
	FUNC_FOLLOW,
	FUNC_FOCUSNP,
	FUNC_REFRESH,
	FUNC_NEXTSRC,
	FUNC_RESETSRC,
};

enum {
	NO_EDGE,
	SUNKEN_EDGE,
	GREY_EDGE,
};

static const char * func_name[] =
{
	"Disabled",
	"Open in external viewer",
	"Open directory",
	"Toggle follow cursor",
	"Focus playing",
	"Refresh",
	"Next source",
	"First source",
};

class cfg_stringlist_t : public cfg_var, public list_t<string8>
{
private:
	const char ** defaults;
public:
	virtual void get_data_raw(stream_writer * p_stream,abort_callback & p_abort) {
		t_uint32 n, m = get_count();
		p_stream->write_lendian_t(m,p_abort);
		for(n=0;n<m;n++) {
			p_stream->write_string(get_item(n),p_abort);
		}
	}
	
	virtual void set_data_raw(stream_reader * p_stream,t_size p_sizehint,abort_callback & p_abort) {
		t_uint32 n,count;
		p_stream->read_lendian_t(count,p_abort);
		remove_all();
		for(n=0;n<count;n++) {
			string8 buf;
			p_stream->read_string(buf,p_abort);
			add_item(buf);
		}
	}
public:
	cfg_stringlist_t(const GUID & p_guid, const char ** p_val) : defaults(p_val), cfg_var(p_guid)
	{
		reset();
	}

	void reset()
	{
		remove_all();
		for (int n = 0; defaults[n] != NULL; n++)
			add_item(defaults[n]);
	}
};

// {4568B002-5CFC-481a-B020-2CD40B5F1D64}
static const GUID cfg_src_list_guid = { 0x4568b002, 0x5cfc, 0x481a, { 0xb0, 0x20, 0x2c, 0xd4, 0xb, 0x5f, 0x1d, 0x64 } };
static const char * cfg_src_list_defaults[] =
{
	"$replace(%_path%,%_filename_ext%,)folder.jpg",
	"-match:$replace(%_path%,%_filename_ext%,)*",
	"-components\\default.png",
	NULL,
};
static cfg_stringlist_t cfg_src_list(cfg_src_list_guid, cfg_src_list_defaults);

// {AE847BFF-97E2-40cb-9EF5-01528818AA71}
static const GUID cfg_centeralbum_guid = { 0xae847bff, 0x97e2, 0x40cb, { 0x9e, 0xf5, 0x1, 0x52, 0x88, 0x18, 0xaa, 0x71 } };
static cfg_bool cfg_centeralbum(cfg_centeralbum_guid,true);

// {3F36BA65-5AE5-4424-8B9B-F6BADBFB1642}
static const GUID cfg_expandalbum_guid = { 0x3f36ba65, 0x5ae5, 0x4424, { 0x8b, 0x9b, 0xf6, 0xba, 0xdb, 0xfb, 0x16, 0x42 } };
static cfg_bool cfg_expandalbum(cfg_expandalbum_guid,true);

// {2F826E17-94FF-43a9-BD4F-6AC83D23FC2A}
static const GUID cfg_shrinkalbum_guid = { 0x2f826e17, 0x94ff, 0x43a9, { 0xbd, 0x4f, 0x6a, 0xc8, 0x3d, 0x23, 0xfc, 0x2a } };
static cfg_bool cfg_shrinkalbum(cfg_shrinkalbum_guid,true);

// {9CC6FD9D-7E6B-44f8-AE37-B6631DC97556}
static const GUID cfg_aspectratio_guid = { 0x9cc6fd9d, 0x7e6b, 0x44f8, { 0xae, 0x37, 0xb6, 0x63, 0x1d, 0xc9, 0x75, 0x56 } };
static cfg_bool cfg_aspectratio(cfg_aspectratio_guid,true);

// {5F3D230E-574B-411e-9B4C-10EBAAB2E846}
static const GUID cfg_pixelborder_guid = { 0x5f3d230e, 0x574b, 0x411e, { 0x9b, 0x4c, 0x10, 0xeb, 0xaa, 0xb2, 0xe8, 0x46 } };
static cfg_bool cfg_pixelborder(cfg_pixelborder_guid,false);

// {C9F9488B-158B-4b80-BE51-3B759CEA5CE4}
static const GUID cfg_selected_guid = { 0xc9f9488b, 0x158b, 0x4b80, { 0xbe, 0x51, 0x3b, 0x75, 0x9c, 0xea, 0x5c, 0xe4 } };
static cfg_bool cfg_selected(cfg_selected_guid,false);

// {017EA865-BBB0-4510-8BDB-537E6B013128}
static const GUID cfg_usebg_guid = { 0x17ea865, 0xbbb0, 0x4510, { 0x8b, 0xdb, 0x53, 0x7e, 0x6b, 0x1, 0x31, 0x28 } };
static cfg_bool cfg_usebg(cfg_usebg_guid,false);

// {12830E6E-02CE-4da4-A1AC-A13B7EFBEA94}
static const GUID cfg_bgcol_guid = { 0x12830e6e, 0x2ce, 0x4da4, { 0xa1, 0xac, 0xa1, 0x3b, 0x7e, 0xfb, 0xea, 0x94 } };
static cfg_int cfg_bgcol(cfg_bgcol_guid,0xFFFFFF);

// {4C23963C-FA5A-4dec-9612-313DEFD51261}
static const GUID cfg_lftclickfunc_guid = { 0x4c23963c, 0xfa5a, 0x4dec, { 0x96, 0x12, 0x31, 0x3d, 0xef, 0xd5, 0x12, 0x61 } };
static cfg_int cfg_lftclickfunc(cfg_lftclickfunc_guid,FUNC_REFRESH);

// {0F8E010C-10E2-4a2f-A58D-ECBD25274B40}
static const GUID cfg_mdlclickfunc_guid = { 0xf8e010c, 0x10e2, 0x4a2f, { 0xa5, 0x8d, 0xec, 0xbd, 0x25, 0x27, 0x4b, 0x40 } };
static cfg_int cfg_mdlclickfunc(cfg_mdlclickfunc_guid,FUNC_OPENEX);

// {9C8A91CB-E377-4861-B954-FCB32905A784}
static const GUID cfg_dblclickfunc_guid = { 0x9c8a91cb, 0xe377, 0x4861, { 0xb9, 0x54, 0xfc, 0xb3, 0x29, 0x5, 0xa7, 0x84 } };
static cfg_int cfg_dblclickfunc(cfg_dblclickfunc_guid,FUNC_FOLLOW);

// {3C90DB14-5D5C-4d7b-8E07-C8EEE941216B}
static const GUID cfg_interpolationmode_guid = { 0x3c90db14, 0x5d5c, 0x4d7b, { 0x8e, 0x7, 0xc8, 0xee, 0xe9, 0x41, 0x21, 0x6b } };
static cfg_int cfg_interpolationmode(cfg_interpolationmode_guid,3);

// {C62D77E8-4962-48c6-AF1E-13ADCC2B51C6}
static const GUID cfg_usecontrol_guid = { 0xc62d77e8, 0x4962, 0x48c6, { 0xaf, 0x1e, 0x13, 0xad, 0xcc, 0x2b, 0x51, 0xc6 } };
static cfg_bool cfg_usecontrol(cfg_usecontrol_guid,false);

// {BADFD1DE-F600-45cc-B7E3-F4DF622D27CF}
static const GUID cfg_minheight_guid = { 0xbadfd1de, 0xf600, 0x45cc, { 0xb7, 0xe3, 0xf4, 0xdf, 0x62, 0x2d, 0x27, 0xcf } };
static cfg_int cfg_minheight(cfg_minheight_guid,0);

// {9DC1A459-A693-44e5-8CCE-9E1AF2ACB65B}
static const GUID cfg_resetsource_guid = { 0x9dc1a459, 0xa693, 0x44e5, { 0x8c, 0xce, 0x9e, 0x1a, 0xf2, 0xac, 0xb6, 0x5b } };
static cfg_bool cfg_resetsource(cfg_resetsource_guid,true);

// {C056F9D2-C906-4827-94A6-C73B2E09F984}
// Not configurable?
//static const GUID cfg_padding_guid = { 0xc056f9d2, 0xc906, 0x4827, { 0x94, 0xa6, 0xc7, 0x3b, 0x2e, 0x9, 0xf9, 0x84 } };
//static cfg_int cfg_padding(cfg_padding_guid,0);

// {A112E31A-FF07-44df-8958-587519313567}
static const GUID cfg_animtime_guid = { 0xa112e31a, 0xff07, 0x44df, { 0x89, 0x58, 0x58, 0x75, 0x19, 0x31, 0x35, 0x67 } };
static cfg_int cfg_animtime(cfg_animtime_guid,300);

// {186B449E-B18C-486e-939A-6DD70D86259F}
static const GUID cfg_edge_style_guid = { 0x186b449e, 0xb18c, 0x486e, { 0x93, 0x9a, 0x6d, 0xd7, 0xd, 0x86, 0x25, 0x9f } };
static cfg_int cfg_edge_style(cfg_edge_style_guid,1);

// {744B6A55-D173-4cb9-A7AE-056CE1EDEF03}
static const GUID cfg_slideffect_guid = 
{ 0x744b6a55, 0xd173, 0x4cb9, { 0xa7, 0xae, 0x5, 0x6c, 0xe1, 0xed, 0xef, 0x3 } };
static cfg_int cfg_slideffect(cfg_slideffect_guid,1);

static inline DWORD exstyle_from_config()
{
	switch (cfg_edge_style)
	{
		case SUNKEN_EDGE: return WS_EX_CLIENTEDGE;
		case GREY_EDGE: return WS_EX_STATICEDGE;
		default: return 0;
	}
}

static bool configchanged=false;
