#include "uie_albumart.h"

#ifdef ENABLE_MATROSKA
DECLARE_COMPONENT_VERSION("Album Art Panel feat. Matroska","0.1516","A Columns_UI extension by Nathan Kallus.\r\nModified by David L to fix redrawing issues and add animation\r\nPorted to Foobar2000 0.9 by G-Lite")
#else
DECLARE_COMPONENT_VERSION("Album Art Panel","0.151","A Columns_UI extension by Nathan Kallus.\r\nModified by David L to fix redrawing issues and add animation\r\nPorted to Foobar2000 0.9 by G-Lite")
#endif

// {5549FA34-673C-4c5b-B6CE-5CDC4813E7B4}
const GUID albumart_control_base::class_guid =
{ 0x5549fa34, 0x673c, 0x4c5b, { 0xb6, 0xce, 0x5c, 0xdc, 0x48, 0x13, 0xe7, 0xb4 } };

uie_albumart::uie_albumart() {
	animshouldstart=animstarted=lastwidth=lastheight=0;
	bufnew=bufold=NULL;
	animating=false;
	b_dblclick = false;
	m_wnd = NULL;
#ifdef ENABLE_MATROSKA
	MatroskaCleanup();
	m_type = type_file;
#endif
}

bool uie_albumart::is_available(const uie::window_host_ptr & p_host) const
{
	return m_host.is_empty() || p_host->get_host_guid() != m_host->get_host_guid();
}

HWND uie_albumart::create_or_transfer_window(HWND wnd_parent, const uie::window_host_ptr & p_host, const ui_helpers::window_position_t & p_position)
{
	if (m_wnd == NULL) {
		static const wchar_t g_class_name[] = L"foo_uie_albumart.dll window class";
		WNDCLASS wc;
		memset(&wc,0,sizeof(wc));
		wc.style = CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS;
		wc.lpfnWndProc = host_proc;
		wc.hInstance = core_api::get_my_instance();
		wc.hCursor = LoadCursor(0,IDC_ARROW);
		wc.hbrBackground = 0;
		wc.lpszClassName = g_class_name;
		RegisterClass(&wc);
		m_wnd = CreateWindowEx(exstyle_from_config(),
			g_class_name,L"foo_uie_albumart",
			WS_CHILD,
			CW_USEDEFAULT,CW_USEDEFAULT,
			0,0,
			wnd_parent,0,core_api::get_my_instance(),this);
	} else {
		SetParent(m_wnd, wnd_parent);
		m_host->relinquish_ownership(m_wnd);
	}
	m_host = p_host;
	return m_wnd;
}

void uie_albumart::destroy_window()
{
#ifdef ENABLE_MATROSKA
	MatroskaCleanup();
#endif
	if (bufold) {
		delete bufold;
		bufold=0;
	}
	if (bufnew) {
		delete bufnew;
		bufnew=0;
	}
	if (bufanim) {
		delete bufanim;
		bufanim=0;
	}
	set_handle(seltrack, 0);
	set_handle(plytrack, 0);
	set_handle(crntrack, 0);
	GdiplusShutdown(gdiplusToken);
	if (m_wnd != NULL) {
		//foo_uie_albumart_var_notify.get_static_instance().remove_window(m_wnd);
		foo_uie_albumart_playlist_notify.get_static_instance().remove_window(m_wnd);
		foo_uie_albumart_playback_notify.get_static_instance().remove_window(m_wnd);
		DestroyWindow(m_wnd);
		m_wnd = NULL;
	}
	m_host.release();
}

LRESULT WINAPI uie_albumart::host_proc(HWND wnd1,UINT msg,WPARAM wp,LPARAM lp){
	uie_albumart * p_this;
	
	if(msg == WM_NCCREATE)
	{
		p_this = (uie_albumart *)((CREATESTRUCT *)(lp))->lpCreateParams; //retrieve pointer to class
		uSetWindowLong(wnd1, GWL_USERDATA, (LPARAM)p_this);//store it for future use
	}else{
		p_this = reinterpret_cast<uie_albumart*>(uGetWindowLong(wnd1,GWL_USERDATA));//if isnt wm_create, retrieve pointer to class
	}
	return p_this ? p_this->on_message(wnd1, msg, wp, lp) : uDefWindowProc(wnd1, msg, wp, lp);
}

void uie_albumart::set_anim_timer() {
	SetTimer(m_wnd,7,10,NULL);
}

LRESULT uie_albumart::on_message(HWND wnd1,UINT msg,WPARAM wp,LPARAM lp) {
	switch(msg)
	{
	case WM_CREATE:
		GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);

		foo_uie_albumart_playlist_notify.get_static_instance().add_window(wnd1);
		foo_uie_albumart_playback_notify.get_static_instance().add_window(wnd1);

		image_locked=false;
		crnsource=nextgoodsource=0;
		m_current_image.source = m_current_image.activesource = m_current_image.subsource = 0;
		m_current_image.mask.reset();
		m_current_image.path.reset();
		m_sources.clear();
		
		if(plytrack.is_empty()){
			metadb_handle_ptr track;
			static_api_ptr_t<playback_control>()->get_now_playing(track);
			if(track.is_valid())
				set_handle(plytrack,track);
		}
		if(seltrack.is_empty()){
			metadb_handle_ptr track;
			static_api_ptr_t<playlist_manager>()->activeplaylist_get_focus_item_handle(track);
			if(track.is_valid())
				set_handle(seltrack,track);
		}
		//if(img.is_empty() && set_image())
		//	redraw();
		if (m_current_image.path.is_empty() && set_image()) {
			redraw();
		}
		break;
	case WM_LBUTTONDBLCLK:
		click_func(cfg_dblclickfunc);
		b_dblclick = true;
		break;
	case WM_LBUTTONUP:
		if(!b_dblclick) {  
		click_func(cfg_lftclickfunc);
		}
		b_dblclick=false;
		break;
	case WM_MBUTTONUP:
		click_func(cfg_mdlclickfunc);
		break;
	case WM_PAINT:
		paint();
		return 0;
		break;
	case WM_TIMER:
		if (wp==7) {
		KillTimer(m_wnd,7);
		paint();
		return 0;
		}
		break;
	case WM_GETMINMAXINFO:
		{
			LPMINMAXINFO mmi = LPMINMAXINFO(lp);
			mmi->ptMinTrackSize.y = cfg_minheight;
		}
		break;
	case WM_FOO_CALLBACK:
		switch(wp)
		{
		case PLAYBACK_NEW_TRACK:
			{
				metadb_handle * p_track = reinterpret_cast<metadb_handle*>(lp);
				metadb_handle_ptr track(p_track);
				if (track.is_valid())
					p_track->service_release();
				set_handle(plytrack,track);
				if(!cfg_selected && !image_locked && set_image(plytrack))
					redraw();
			}
			break;
		case PLAYBACK_STOP:
			if(lp!=playback_control::stop_reason_starting_another && !image_locked && seltrack.is_valid() && set_image(seltrack))
				redraw();
			break;
		case PLAYLIST_FOCUS_CHANGE:
			{
				metadb_handle_ptr track;
				static_api_ptr_t<playlist_manager>()->activeplaylist_get_item_handle(track, (t_size)lp);
				if(track.is_valid())
					set_handle(seltrack,track);
			}
			if (cfg_selected || !static_api_ptr_t<playback_control>()->is_playing()){
				if(seltrack.is_valid())
				{
					if(!image_locked && set_image(seltrack))
						redraw();
				}
			}
			break;
		case VARIABLE_CHANGE:
			{
				GUID * guid = reinterpret_cast<GUID *>(lp);
				if(*guid == cfg_minheight_guid){
					m_host->on_size_limit_change(wnd1,uie::size_limit_minimum_height);
				}else if(*guid == cfg_usecontrol_guid){
					lock(false);
					if(set_image())
						redraw();
				}else if(*guid == cfg_src_list_guid){
					m_current_image.source = 0;//crnsource=0;
				}
			}
			if(!image_locked)
				set_image();
			redraw();
			break;
		}
		break;
	}
	return uDefWindowProc(wnd1, msg, wp, lp);
}

class albumart_menu_functions : public uie::menu_node_command_t
{
private:
	int func_idx;
	cfg_bool * checked;
public:
	bool get_display_data(pfc::string_base & p_out,unsigned & p_state) const
	{
		p_out = func_name[func_idx];
		if (checked != NULL && *checked)
			p_state = state_checked;
		else
			p_state = 0;
		return true;
	}
	bool get_description(pfc::string_base & p_out) const
	{
		return false;
	}
	virtual void execute()
	{
		foo_uie_albumart.get_static_instance().click_func(func_idx);
	}

	albumart_menu_functions(int p_func_idx, cfg_bool * p_checked = NULL)
	{
		func_idx = p_func_idx;
		checked = p_checked;
	}
};

class albumart_menu_preferences : public uie::menu_node_command_t
{
public:
	bool get_display_data(pfc::string_base & p_out,unsigned & p_state) const
	{
		p_out = "Preferences...";
		p_state = 0;
		return true;
	}
	bool get_description(pfc::string_base & p_out) const
	{
		return false;
	}
	virtual void execute()
	{
		static_api_ptr_t<ui_control>()->show_preferences(albumart_preferences_guid);
	}
};

void uie_albumart::get_menu_items (uie::menu_hook_t & p_hook)
{
	uie::menu_node_ptr item;
	for(int i=1;i<tabsize(func_name);i++)
	{
		if (i == FUNC_FOLLOW)
			item = new albumart_menu_functions(i, &cfg_selected);
		else
			item = new albumart_menu_functions(i);
		p_hook.add_node(item);
	}
	item = new uie::menu_node_separator_t();
	p_hook.add_node(item);
	item = new albumart_menu_preferences();
	p_hook.add_node(item);
}

void uie_albumart::redraw()
{
	RedrawWindow(m_wnd, NULL, NULL, RDW_INVALIDATE | RDW_ERASE);
}

void uie_albumart::click_func(unsigned func)
{
	switch(func){
	case FUNC_OPENEX:
		func_OpenEx();
		break;
	case FUNC_OPENDIR:
		func_OpenDir();
		break;
	case FUNC_FOLLOW:
		func_ToggleFollow();
		break;
	case FUNC_FOCUSNP:
		func_FocusPlaying();
		break;
	case FUNC_REFRESH:
		func_Refresh();
		break;
	case FUNC_NEXTSRC:
		func_NextSource();
		break;
	case FUNC_RESETSRC:
		func_ResetSource();
		break;
	}
}

void uie_albumart::func_OpenEx()
{
	if(!img.is_empty())
		uShellExecute(0,"open",img,0,0,SW_SHOW);
}

void uie_albumart::func_OpenDir()
{
	if(crntrack.is_valid()){
		string8 temp;
		crntrack->format_title(NULL,temp,"$replace(%_path%,%_filename_ext%,)",NULL);
		uShellExecute(0,"open",temp,0,0,SW_SHOW);
	}
}

void uie_albumart::func_ToggleFollow()
{
	cfg_selected=!cfg_selected;
}

void uie_albumart::func_FocusPlaying()
{
	static_api_ptr_t<playlist_manager>()->highlight_playing_item();
}

void uie_albumart::func_Refresh()
{
	if(!image_locked)
		set_image();
	redraw();
}

/*
void uie_albumart::func_NextSource() //WigBaM's modified code
{
unsigned n, oldsource; // antihang; // very helfpful for testing purposes;]
oldsource = nextgoodsource; //we save index of *currently displaying* image (which is set in set_image)
if(crntrack.is_valid()){
 unsigned m = cfg_src_list.get_count();
 n=crnsource+1;
// antihang = 0; // testing
 while(1)
 {
  if(n>=m) // ATTENTION: Changed this to m (was m-1). Otherwise we would skip last entry
   n=0;
  for(;cfg_src_list[n].find_first("-")==0;n++){ // get next source
   if(n==crnsource)
    break;
   else if(n>=m-1)
    n=-1;
  }
  if(n==crnsource)  // in case we are back to where
   break;    // we've starterd

  if(set_image(crntrack,n))  // ok, so do we have _different_ image then?
  {
   crnsource=nextgoodsource; // yes, let it be our current source then
   redraw();
   break;
  }
  else if(nextgoodsource!=oldsource) // nah, image path is the same. Now let's check whether
  {         // it is from different source entry. Weird, but happens.
           // purely for additional bug support :)
   crnsource=nextgoodsource; // OH YEAH, BABY! (c) Austin + YELLO
   break;					
  }

// So far we have found the same image with same index, so let's see
// whether we've scanned every entry and returned to the initial source
  if(nextgoodsource < n)
   break;

  n=nextgoodsource+1;  // Otherwise let's see what's next

////// TESTING
/*   antihang++;
  if(antihang>50)
  {
   console::output(1, "hanged :(((");
   break;
  }//*:/
 }
}
}
*/
void uie_albumart::func_NextSource()
{
	string8 temp;
	if(crntrack.is_valid()){
		bool b_redraw = false;

		m_temp_image = m_current_image;
		m_temp_image.activesource = m_temp_image.source;	// start
		m_temp_image.subsource++;

		if(set_image(m_current_image, &m_temp_image))
			b_redraw = true;

		if(m_temp_image.path.is_empty()) {
			return;	//nothing found
		} else if(stricmp_utf8(m_current_image.mask, m_temp_image.mask) != 0) {
			getImageMatches(m_sources, m_temp_image.mask);
			m_temp_image.subsource = 0;
		}

		assign_and_reset_if_only();
		m_is_nextsource = true;
		if(b_redraw) redraw();
	}
}

void uie_albumart::assign_and_reset_if_only() //this is purely a helper function 
{
	m_current_image = m_temp_image;
	if(/*!cfg_resetifonly || */m_current_image.source == 0) return;
	m_temp_image.activesource++;
	m_temp_image.subsource = 0;
	m_temp_image.mask.reset();
	m_temp_image.path.reset();
	if(!set_image(m_temp_image, &m_temp_image))	m_current_image.source = 0;
}

void uie_albumart::func_ResetSource()
{
	//if(crntrack.is_valid() && set_image(crntrack,(crnsource=0)))
	//	redraw();
	if(crntrack.is_valid()) {
		m_current_image.source = m_current_image.activesource = 0;
		m_current_image.subsource = -1;
		m_current_image.mask.reset();
		m_sources.clear();
		func_NextSource();
		m_is_nextsource = false;
	}
}

bool uie_albumart::set_image() {
	return set_image(!static_api_ptr_t<playback_control>()->is_playing()?seltrack:cfg_selected?seltrack:plytrack);
}

bool uie_albumart::set_image(unsigned fstsource) {
	return false;
	//return set_image(!static_api_ptr_t<playback_control>()->is_playing()?seltrack:cfg_selected?seltrack:plytrack, fstsource);
}
/*
bool uie_albumart::set_image(metadb_handle_ptr track){
	if(!cfg_resetsource)
		return set_image(track,crnsource);
	else{
		string8 img_org=img;
		if(set_image(track,crnsource)){
			crnsource=0;
			img=img_org;
			return set_image(track,crnsource);
		}else
			return false;
	}
}
*/

bool uie_albumart::set_image(metadb_handle_ptr track){
	bool b_redraw = false;
	set_handle(crntrack, track);
	if (crntrack.is_valid()) {
		if (!cfg_resetsource) {
			int source = m_current_image.source;
			m_current_image.subsource--;
			func_NextSource();
			m_is_nextsource = false;
			if(!(/*cfg_resetifonly&& */m_current_image.source==0)) m_current_image.source = source;
			return false;
		}
		m_is_nextsource = false;
		m_temp_image = m_current_image;
		m_temp_image.activesource = m_current_image.source;
		b_redraw = set_image(m_temp_image, &m_temp_image);
		if (b_redraw) {
			if (m_temp_image.source != 0) { func_ResetSource(); return false; }
			if (stricmp(m_current_image.mask, m_temp_image.mask)) {
				getImageMatches(m_sources, m_temp_image.mask);
				m_temp_image.subsource = 0;
			}
			assign_and_reset_if_only();
			return true;
		} else if (m_temp_image.activesource != m_current_image.activesource) {
			func_ResetSource();
		}
	}
	return b_redraw;
}

bool uie_albumart::set_image(image_data current, image_data *next)
{
	int source_list_count = cfg_src_list.get_count();
	next->source = next->activesource;
	next->path.reset();
	current.mask.reset();
	if (!(next->subsource < m_sources.size()) && next->subsource > 0) {
		next->subsource = 0;
		next->mask.reset();
		next->source = next->activesource = current.activesource+1;
	}

	if (next->activesource >= source_list_count) {	// ATTENTION: Changed this to m (was m-1). Otherwise we would skip last entry
		next->activesource=0;
	}
	for (; string8(cfg_src_list[next->activesource]).find_first("-") == 0; next->activesource++) {
		if (next->activesource == current.source) {
			break;
		} else if (next->activesource >= source_list_count-1) {
			next->activesource = -1;
		}
	}

	for (; next->activesource < source_list_count; next->activesource++) {
		string8 pattern = cfg_src_list[next->activesource];
		if (pattern.find_first("-") == 0) {
			pattern.remove_chars(0, 1);
		} else if (next->source != 0) {
			next->source = next->activesource; //we want to stay at 0 for 1st image
		}
		bool match = false;
		SourceType type = type_file;
		if (pattern.find_first("match:") == 0) {
			match = true;
			pattern.remove_chars(0, 6);
			type = type_file;
		} else if (pattern.find_first("matroska:")==0) {
			// matroska:
			string8 tmp_pattern = pattern;
			pattern.remove_chars(0, 9);
			string8 old_filepath = m_filepath;
			m_filepath = crntrack->get_path();
			if (stricmp_utf8(string_extension(m_filepath), "MKA") == 0 || stricmp_utf8(string_extension(m_filepath), "MKV") == 0) {
				if (stricmp_utf8(old_filepath, m_filepath) != 0) {
					service_ptr_t<file> p_file;
					Matroska::MatroskaAudioParser* parser = NULL;
					if (filesystem::g_exists(m_filepath, m_abort)) {
						filesystem::g_open_read(p_file, m_filepath, m_abort);
						parser = new Matroska::MatroskaAudioParser(p_file, m_abort);
					}
					if (parser != NULL) {
						parser->Parse(true);
						if (parser->GetAttachmentList().size() > 0) {
							m_attachment_list.swap(parser->GetAttachmentList());
							type = type_matroska;
						}
						delete parser;
						parser = NULL;
					}
				} else {
					type = type_matroska;
				}
			} else {
				pattern = tmp_pattern;
			}
			match = true;
		} else if (pattern.find_first("archive:")==0) {
			// archive:
			string8 tmp_pattern = pattern;
			pattern.remove_chars(0, 8);
			string8 tmp_archive, tmp_file;
			if (archive_impl::g_parse_unpack_path(crntrack->get_path(), tmp_archive, tmp_file)) {
				type = type_archive;
			} else {
				pattern = tmp_pattern;
			}
			match = true;
		} else {
			type = type_file;
		}
		string8 file;
		crntrack->format_title(NULL, file, pattern, NULL);
		if (type == type_matroska) {
			file = CreateMatroskaPath(file, crntrack->get_path());
		} else if (type == type_archive) {
			string8 tmp, tmp_archive, tmp_file;
			archive_impl::g_parse_unpack_path(crntrack->get_path(), tmp_archive, tmp_file);
			archive_impl::g_make_unpack_path(tmp, tmp_archive, file, string_extension(tmp_archive));
			file = tmp;
		}
		current.mask = file;
		if (match) {
			if (stricmp(file, next->mask) == 0) {
				file = m_sources.at(next->subsource);
			} else {
				findFirstImageMatch(file, &file);
			}
		}
		if (!file.is_empty()) {
			if (IsBitmap(file)) {
				next->path = file;
				next->mask.set_string(current.mask);
				break;
			}
		}
		if (next->activesource == current.source-1 || (next->activesource == source_list_count-1 && current.source == 0)) {
			break;
		}
		if (next->activesource == source_list_count-1) {
			next->activesource = -1;
		}
	}

	animshouldstart = GetTickCount();
	return next->path.is_empty() ? !current.path.is_empty() : (strcmp_partial(current.path, next->path) != 0);
}
#if 0
bool uie_albumart::set_image(metadb_handle_ptr track, unsigned fstsource)
{
	unsigned n,m = cfg_src_list.get_count();
	if(track.is_empty() || fstsource>=m) return false;
	string8 img_org=img;
	img.reset();
	set_handle(crntrack,track);
	if(m==0) {
		animshouldstart=GetTickCount();
		return true;
	}
	for(n=fstsource;n<m;n++)
	{
		string8 pattern=cfg_src_list[n];
		if(pattern.find_first("-")==0)
			pattern.remove_chars(0,1);
		bool match=false;
		if(pattern.find_first("match:")==0){
			match=true;
			pattern.remove_chars(0,6);
		}
#ifdef ENABLE_MATROSKA
		bool matroska = false;
		// matroska:
		if (pattern.find_first("matroska:")==0) {
			pattern.remove_chars(0, 9);
			string8 old_filepath = m_filepath;
			crntrack->format_title(NULL, m_filepath, "%_path%", NULL);
			m_fileext = string_extension(m_filepath);
			if (stricmp_utf8(m_fileext, "MKA") == 0) {
				if (stricmp_utf8(old_filepath, m_filepath) != 0) {
					service_ptr_t<file> p_file;
					if (filesystem::g_exists(m_filepath, m_abort)) {
						filesystem::g_open_read(p_file, m_filepath, m_abort);
						m_parser = new Matroska::MatroskaAudioParser(p_file, m_abort);
					}
					if (m_parser != NULL) {
						m_parser->Parse(true);
						if (m_parser->GetAttachmentList().size() > 0) {
							m_attachment_list = m_parser->GetAttachmentList();
							matroska = true;
						}
						delete m_parser;
						m_parser = NULL;
					}
				} else {
					matroska = true;
				}
			}
		}
		// archive:
		bool archive = false;
		if (pattern.find_first("archive:")==0) {
			pattern.remove_chars(0, 8);
			archive = true;
		}
#endif
		string8 file;
		crntrack->format_title(NULL,file,pattern,NULL);
		if(match){
			file=::findFirstImageMatch(file);
		}
#if ENABLE_MATROSKA
		if (matroska) {
			file = MatroskaFindFirstImageMatch(file);
		}
		if (archive) {
			file = ArchiveFindFirstImageMatch(file);
		}
		if(!file.is_empty()){
			if (matroska) {
				if (m_stream != NULL) m_stream->Release();
				m_stream = MatroskaReadAttachment(file);
				Bitmap bmp(m_stream);
				if(bmp.GetHeight()){
					m_attachment_filename = file;
					file = m_filepath;
					img=file;
					nextgoodsource=n;
					m_type = type_matroska;
					break;
				}
			} else if (archive) {
				if (m_stream != NULL) m_stream->Release();
				m_stream = ArchiveReadAttachment(file);
				Bitmap bmp(m_stream);
				if(bmp.GetHeight()){
					m_attachment_filename = file;
					img=file;
					nextgoodsource=n;
					m_type = type_archive;
					break;
				}
			} else {
				Bitmap bmp(stringcvt::string_wide_from_utf8(file.get_ptr()));
				if(bmp.GetHeight()){
					MatroskaCleanup();
					m_attachment_filename.reset();
					m_filepath.reset();
					img=file;
					nextgoodsource=n;
					m_type = type_file;
					break;
				}
			}
#else
		if(!file.is_empty()){
			Bitmap bmp(stringcvt::string_wide_from_utf8(file.get_ptr()));
			if(bmp.GetHeight()){
				 img=file;
				 nextgoodsource=n;
				 break;
			}
#endif
		}
		if(n==fstsource-1 || (n==m-1 && fstsource==0)) break;
		if(n==m-1) n=-1;
	}
	animshouldstart=GetTickCount();
	return img.is_empty() ? !img_org.is_empty() : strcmp_partial(img_org,img)!=0 ;
}
#endif

void uie_albumart::paint()
{
	RECT wndrect_temp;
	if(GetClientRect(m_wnd,&wndrect_temp)){
		Graphics gfx(m_wnd);
		Rect wndrect(
			wndrect_temp.left,
			wndrect_temp.top,
			wndrect_temp.right-wndrect_temp.left,
			wndrect_temp.bottom-wndrect_temp.top
		);
		
		bool noimage=(!bufnew);
		bool imagechanged=(strcmp(bufimg.get_ptr(),m_current_image.path.get_ptr())!=0);
		bool sizechanged=bufnew?(bufnew->GetWidth()!=wndrect.Width||bufnew->GetHeight()!=wndrect.Height):true;
		if (noimage||imagechanged||sizechanged||configchanged) {
			SetWindowLong(m_wnd, GWL_EXSTYLE, exstyle_from_config());
			SetWindowPos(m_wnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE|SWP_NOZORDER|SWP_FRAMECHANGED);

			if (sizechanged||noimage) {
				if (animating) animating=false;
				if (bufold) {
					delete bufold;
					bufold=NULL;
				}
				if (bufanim) {
					delete bufanim;
					bufanim=NULL;
				}
				if (bufnew) {
					delete bufnew;
					bufnew=NULL;
				}
				bufnew = new Bitmap(wndrect.Width, wndrect.Height, &gfx);
				bufold = new Bitmap(wndrect.Width, wndrect.Height, &gfx);
				if (cfg_animtime!=0) {
					bufanim = new Bitmap(wndrect.Width, wndrect.Height, &gfx);
				}
			} else if (configchanged) {
				if (bufanim&&cfg_animtime==0) {
					delete bufanim;
					bufanim=NULL;
				} else if (!bufanim&&cfg_animtime!=0) {
					bufanim = new Bitmap(wndrect.Width, wndrect.Height, &gfx);
				}
			}

			if (configchanged) configchanged=false;
			Bitmap *temp;
			temp=bufold;
			bufold=bufnew;
			bufnew=temp;
		
			Graphics bufgfx(bufnew);
			bufgfx.Clear(cfg_usebg?Color((cfg_bgcol) & 0xFF,(cfg_bgcol>> 8) & 0xFF,(cfg_bgcol>> 16) & 0xFF):Color::Black);

			if(!m_current_image.path.is_empty()){
#ifdef ENABLE_MATROSKA
				IStream* stream = GetImageStream(m_current_image.path);
				Bitmap bmp(stream);
#else
				Bitmap bmp(stringcvt::string_wide_from_utf8(img.get_ptr()));
#endif
				/*
				if(cfg_padding>0){
					wndrect.Height-=2*cfg_padding;
					wndrect.Width-=2*cfg_padding;
					wndrect.X+=cfg_padding;
					wndrect.Y+=cfg_padding;
				}
				*/
				RectF rect(wndrect.X,wndrect.Y,bmp.GetWidth(),bmp.GetHeight());
				if (
					(cfg_expandalbum && rect.GetRight() < wndrect.GetRight()  &&  rect.GetBottom() < wndrect.GetBottom())  ||
					(cfg_shrinkalbum  &&  (rect.GetRight() > wndrect.GetRight()  ||  rect.GetBottom() > wndrect.GetBottom())))
				{
					if(!cfg_aspectratio){
						rect.Width=wndrect.GetRight();
						rect.Height=wndrect.GetBottom();
					}
					else if( (wndrect.GetRight()-rect.GetRight()) < (wndrect.GetBottom()-rect.GetBottom()) ){
						rect.Height*=((double)wndrect.GetRight()/rect.GetRight());
						rect.Width=wndrect.GetRight();
					}else{
						rect.Width*=((double)wndrect.GetBottom()/rect.GetBottom());
						rect.Height=wndrect.GetBottom();
					}
				}
				if(cfg_centeralbum)
					rect.Offset((double)(wndrect.GetRight()-rect.GetRight())/2,(double)(wndrect.GetBottom()-rect.GetBottom())/2);
				if(cfg_pixelborder){
					rect.Width--;
					rect.Height--;
					bufgfx.DrawRectangle(& Pen(Color(0,0,0)),rect);
					rect.X++;
					rect.Y++;
					rect.Width--;
					rect.Height--;
				}
				bufgfx.SetInterpolationMode(cfg_interpolationmode==2?InterpolationModeHighQuality:
						cfg_interpolationmode==1?InterpolationModeDefault:
						cfg_interpolationmode==0?InterpolationModeLowQuality:
						InterpolationModeHighQualityBicubic);
				int srcWidth=bmp.GetWidth(),srcHeight=bmp.GetHeight(),srcX=0,srcY=0;
				/*
				if(cfg_padding<0 && abs(cfg_padding)<bmp.GetWidth() && abs(cfg_padding)<bmp.GetHeight()){
					srcWidth+=2*cfg_padding;
					srcHeight+=2*cfg_padding;
					srcX-=cfg_padding;
					srcY-=cfg_padding;
				}
				*/
				bufgfx.DrawImage(
					&bmp,
					rect,  // destination rectangle 
					srcX, srcY,        // upper-left corner of source rectangle
					srcWidth,       // width of source rectangle
					srcHeight,      // height of source rectangle
					UnitPixel);
				StreamCleanup(stream);
			}
			bufimg = m_current_image.path;
		
			// bufnew updated. time to fade in the changes.
			DWORD now=GetTickCount();
			if (cfg_animtime!=0&&!sizechanged&&(animshouldstart+700)>now) {
				animating=true;
				animstarted=GetTickCount();
				set_anim_timer();
			}
		}

		Rect dest(wndrect_temp.left,
			wndrect_temp.top,
			wndrect_temp.right-wndrect_temp.left,
			wndrect_temp.bottom-wndrect_temp.top
		);
		if (bufnew&&!animating) {
			gfx.DrawImage(bufnew, 0, 0);
		} else if (animating&&bufnew&&bufanim) {
			float opacityold=0.0f;
			float opacitynew=1.0f;
			int doneness=GetTickCount();
			if (doneness>(animstarted+cfg_animtime)) {
				animating=false;
				opacityold=0.0f;
				opacitynew=1.0f;
			} else if (animstarted<=doneness&&cfg_animtime) {
				opacitynew=(float)(doneness-animstarted)/(float)cfg_animtime;
				opacityold=1.0f-opacitynew;
			}
			{
				Graphics animbuf(bufanim);
				animbuf.Clear(cfg_usebg?Color((cfg_bgcol) & 0xFF,(cfg_bgcol>> 8) & 0xFF,(cfg_bgcol>> 16) & 0xFF):Color::Black);
				ColorMatrix cm;
				ZeroMemory(&cm,sizeof(ColorMatrix));
				cm.m[0][0] = 1.0f;
				cm.m[1][1] = 1.0f;
				cm.m[2][2] = 1.0f;
				cm.m[3][3] = opacityold;

				ImageAttributes ia;
				ia.SetColorMatrix(&cm);
				if(m_is_nextsource && cfg_slideffect) {
					float cor1=1.0;
					float cor2=0.1*(opacityold+0.35);
					Rect temp_rect(dest);
					temp_rect.Width *= opacityold*cor1;
					animbuf.DrawImage(bufold,temp_rect,dest.Width-temp_rect.Width,0,temp_rect.Width,dest.Height,UnitPixel,&ia);
					cm.m[3][3] = opacitynew;
					ia.SetColorMatrix(&cm);
					temp_rect.X += temp_rect.Width*cor2;
					temp_rect.Width = dest.Width-temp_rect.X;
					animbuf.DrawImage(bufnew,temp_rect,0,0,temp_rect.Width,dest.Height,UnitPixel,&ia);
				} else {
					animbuf.DrawImage(bufold,dest,0,0,dest.Width,dest.Height,UnitPixel,&ia);
					cm.m[3][3] = opacitynew;
					ia.SetColorMatrix(&cm);
					animbuf.DrawImage(bufnew,dest,0,0,dest.Width,dest.Height,UnitPixel,&ia);
				}
			}
			gfx.DrawImage(bufanim,0,0);
			if (animating) set_anim_timer();
		}
		else gfx.Clear(cfg_usebg?Color((cfg_bgcol) & 0xFF,(cfg_bgcol>> 8) & 0xFF,(cfg_bgcol>> 16) & 0xFF):Color::Black);

		ValidateRect(m_wnd,NULL);
	}
}

void uie_albumart::lock(bool l){
	image_locked=l;
}

bool uie_albumart::is_locked(){
	return image_locked;
}

#ifdef ENABLE_MATROSKA
string8 uie_albumart::CreateMatroskaPath(string8 attachment, string8 path) {
	string8 result;
	result.set_string("matroska://");
	result.add_string(path);
	result.add_string("|");
	result.add_string(attachment);
	return result;
}

void uie_albumart::getImageMatches(std::vector<string8> &output, string8 file, string8* single_result)
{
	if (single_result == NULL) output.clear();
	if (file.is_empty()) return;
	string8	mask = file;
	if (GetSourceType(file) == type_file) {
		file.replace_char('/','\\');
		file.replace_char('|','\\');
	}
	if (!wildcard_helper::has_wildcards(file)) {
		if (IsBitmap(file)) {
			if(single_result != NULL) {
				single_result->set_string(file);
			} else {
				output.push_back(file);
			}
		}
		return;
	}
	string8 base_path = file;
	switch (GetSourceType(file)) {
	case type_file:
		file.truncate(file.find_first('\\', file.find_first("*")));
		base_path.truncate(1+base_path.find_last('\\'));
		WIN32_FIND_DATA data;
		HANDLE hFind;
		if ((hFind = FindFirstFile(stringcvt::string_wide_from_utf8(file), &data)) == INVALID_HANDLE_VALUE) {
			FindClose(hFind);
			return;
		}
		do {
			if (stricmp_utf8(stringcvt::string_utf8_from_wide(data.cFileName), ".") != 0 &&
				stricmp_utf8(stringcvt::string_utf8_from_wide(data.cFileName),"..") != 0)
			{
				string8 path = base_path;
				path.add_string(stringcvt::string_utf8_from_wide(data.cFileName));
				string8 pattern = mask;
				pattern.remove_chars(0, file.length());
				path.add_string(pattern);
				if(!wildcard_helper::has_wildcards(path) && wildcard_helper::test(path, mask)) {
					if (IsBitmap(path)) {
						if(single_result != NULL) {
							single_result->set_string(path.get_ptr());
							break;
						} else {
							output.push_back(path);
						}
					}
				}
			}
		} while (FindNextFile(hFind, &data));
		FindClose(hFind);
		break;
	case type_matroska:
		base_path.truncate(base_path.find_last("|"));
		for (int i = 0; i != m_attachment_list.size(); ++i) {
			string8 name = stringcvt::string_utf8_from_wide(m_attachment_list.at(i).FileName.c_str());
			mask.remove_chars(0, mask.find_last("|")+1);
			if (wildcard_helper::test(name, mask)) {
				base_path.add_string("|");
				base_path.add_string(name);
				if (IsBitmap(base_path)) {
					if(single_result != NULL) {
						single_result->set_string(base_path.get_ptr());
						break;
					} else {
						output.push_back(base_path);
					}
				}
			}
		}
		break;
	case type_archive:
		break;
	}
//	if(output->get_count() == 1) output->delete_all();
	return;
}

void uie_albumart::findFirstImageMatch(string8 p, string8* output)
{
	std::vector<string8> dummy;
	getImageMatches(dummy, p, output);
}

SourceType uie_albumart::GetSourceType(string8 name)
{
	if (stricmp_utf8_partial(name, "matroska://", 11) == 0) {
		return type_matroska;
	} else if (stricmp_utf8_partial(name, "unpack://", 9) == 0) {
		return type_archive;
	} else {
		return type_file;
	}
}

void uie_albumart::OpenFile(service_ptr_t<file> & file, string8 name)
{
	if (IsFileExists(name)) {
		if (GetSourceType(name) == type_archive) {
			archive::g_open_read(file, name, m_abort);
		} else {
			filesystem::g_open_read(file, name, m_abort);
		}
	}
}

bool uie_albumart::IsFileExists(string8 path)
{
	if (wildcard_helper::has_wildcards(path)) return false;
	if (GetSourceType(path) == type_archive) {
		if (archive::g_is_recognized_path(path)) {
			if (archive::g_exists(path, m_abort)) {
				return true;
			}
		}
	} else {
		if (filesystem::g_is_recognized_path(path)) {
			if (filesystem::g_exists(path, m_abort)) {
				return true;
			}
		}
	}
	return false;
}

bool uie_albumart::IsBitmap(string8 name)
{
	if (IsFileExists(name) && GetSourceType(name) == type_file) {
		return IsBitmapFromFile(name);
	}
	IStream* stream = GetImageStream(name);
	if (stream == NULL) return false;
	Bitmap bmp(stream);
	bool result = false;
	if (bmp.GetHeight()) {
		result = true;
	}
	StreamCleanup(stream);
	return result;
}

bool uie_albumart::IsBitmapFromFile(string8 path)
{
	Bitmap bmp(stringcvt::string_wide_from_utf8(path.get_ptr()));
	if (bmp.GetHeight()) {
		return true;
	} else {
		return false;
	}
}

IStream* uie_albumart::GetImageStream(string8 filename)
{
	Matroska::MatroskaAttachment attachment;
	SourceType type = GetSourceType(filename);
	if (type == type_matroska) {
		filename.remove_chars(0, 11);
		string8 attachment_name = filename;
		attachment_name.remove_chars(0, attachment_name.find_last("|")+1);
		for (int i = 0; i != m_attachment_list.size(); ++i) {
			if (stricmp_utf8(stringcvt::string_utf8_from_wide(m_attachment_list.at(i).FileName.c_str()), attachment_name) == 0) {
				attachment = m_attachment_list.at(i);
			}
		}
		filename.truncate(filename.find_first("|"));
	}
	service_ptr_t<file> p_file;
	OpenFile(p_file, filename);
	if (p_file == NULL) return NULL;
	int filesize = 0;
	if (type == type_matroska) {
		p_file->seek(attachment.SourceStartPos, m_abort);
		filesize = attachment.SourceDataLength;
	} else {
		filesize = p_file->get_size(m_abort);
	}
	hGlobal = GlobalAlloc(GHND, filesize);
	if (hGlobal == NULL) return NULL;
	void* p_buffer = GlobalLock(hGlobal);
	if (p_buffer == NULL) {
		GlobalFree(hGlobal);
		return NULL;
	}
	p_file->read(p_buffer, filesize, m_abort);
	GlobalUnlock(hGlobal);
	IStream* stream;
	HRESULT hr = CreateStreamOnHGlobal(hGlobal, false, &stream);
	if (SUCCEEDED(hr) == FALSE) {
		StreamCleanup(stream);
		return NULL;
	}
	return stream;
}

string8 uie_albumart::MatroskaFindFirstImageMatch(string8 p)
{
	for (int i = 0; i != m_attachment_list.size(); ++i) {
		string8 name = stringcvt::string_utf8_from_wide(m_attachment_list.at(i).FileName.c_str());
		if (wildcard_helper::has_wildcards(p)) {
			if (wildcard_helper::test(name, p)) {
				return name;
			}
		} else {
			if (stricmp_utf8(name, p) == 0) {
				return name;
			}
		}
	}
	return NULL;
}

string8 uie_albumart::ArchiveFindFirstImageMatch(string8 p)
{
	string8 archive_filepath, p_filepath, p_fileext;
	p_filepath = crntrack->get_path();
	crntrack->format_title(NULL, p_filepath, "$replace(%_path%,|%_filename_ext%,)", NULL);
	p_fileext = string_extension(p_filepath);
	archive_impl::g_make_unpack_path(archive_filepath, p_filepath, p, p_fileext);
	if (archive::g_is_recognized_path(archive_filepath)) {
		if (archive::g_exists(archive_filepath, m_abort)) {
			return archive_filepath;
		}
	}
	return NULL;
}

void uie_albumart::StreamCleanup(IStream* stream)
{
	if (stream != NULL) {
		stream->Release();
	}
	if (hGlobal != NULL) {
		GlobalFree(hGlobal);
		hGlobal = NULL;
	}
}

void uie_albumart::MatroskaCleanup(bool p_all)
{
	if (!m_attachment_list.empty() && p_all) {
		m_attachment_list.clear();
	}
}
#endif
