#include <windows.h>

#include "../SDK/foobar2000.h"
#include "../helpers/helpers.h"
#include "../columns_ui-sdk/ui_extension.h"
#include <GdiPlus.h>
using namespace Gdiplus;
//#pragma comment(lib, "gdiplus.lib")
using namespace pfc;
#include "resource.h"
#ifdef ENABLE_MATROSKA
#include "matroska_parser.h"
#endif
#include "albumart_helpers.h"
#include "albumart_vars.h"
#include "albumart_notifiers.h"
// This had to be moved below foo_uie_albumart
//#include "albumart_config.h"
const enum SourceType {
	type_file, type_matroska, type_archive
};

class uie_albumart : public uie::window {
protected:
	metadb_handle_ptr seltrack, plytrack, crntrack;
	unsigned crnsource,nextgoodsource;
	string8 img;
	string8 bufimg;
	int lastwidth,lastheight;
	bool image_locked;
	Bitmap* bufnew;
	Bitmap* bufold;
	Bitmap* bufanim;
	bool animating;
	DWORD animstarted,animshouldstart;
	bool b_dblclick;

	HWND m_wnd;
	uie::window_host_ptr m_host;
	GdiplusStartupInput gdiplusStartupInput;
	ULONG_PTR gdiplusToken;

#ifdef ENABLE_MATROSKA
	void assign_and_reset_if_only();
	SourceType GetSourceType(string8 name);
	string8 CreateMatroskaPath(string8 attachment, string8 path);
	void OpenFile(service_ptr_t<file> & file, string8 name);
	void findFirstImageMatch(string8 p, string8* output);
	void getImageMatches(std::vector<string8> & output, string8 file, string8* single_result = NULL);
	IStream* GetImageStream(string8 filename);
	bool IsFileExists(string8 path);
	bool IsBitmapFromFile(string8 path);
	bool IsBitmap(string8 name);
	void StreamCleanup(IStream* stream);
	HGLOBAL hGlobal;
	string8 MatroskaFindFirstImageMatch(string8 p);
	void MatroskaCleanup(bool p_all = true);
	string8 ArchiveFindFirstImageMatch(string8 p);
	std::vector<Matroska::MatroskaAttachment> m_attachment_list;
	string8 m_filepath;
	abort_callback_impl m_abort;

	struct image_data {
		string8 mask, path;
		int source, activesource, subsource;
	} m_current_image, m_temp_image; //current & temp
	std::vector<string8> m_sources;
	bool m_is_nextsource;
	SourceType m_type;
#endif

public:
	uie_albumart();
	virtual const GUID & get_extension_guid() const {
		// {E32B3859-B6BC-48e1-8370-3D5F5FA00A60}
		static const GUID guid = 
		{ 0xe32b3859, 0xb6bc, 0x48e1, { 0x83, 0x70, 0x3d, 0x5f, 0x5f, 0xa0, 0xa, 0x60 } };
		return guid;
	}
	virtual void get_name(string_base & out) const { out = "Album Art"; }
	virtual void get_category(string_base & out) const {out = "Panels";	}
	virtual void get_menu_items (uie::menu_hook_t & p_hook);

	virtual unsigned get_type() const { return uie::type_panel | uie::type_toolbar; }
	virtual bool is_available(const uie::window_host_ptr & p_host) const;
	virtual HWND create_or_transfer_window(HWND wnd_parent, const uie::window_host_ptr & p_host, const ui_helpers::window_position_t & p_position);
	virtual void destroy_window();
	virtual HWND get_wnd() const {return m_wnd;}

	static LRESULT WINAPI host_proc(HWND wnd,UINT msg,WPARAM wp,LPARAM lp);
	LRESULT WINAPI on_message(HWND wnd,UINT msg,WPARAM wp,LPARAM lp);

	void paint();
	void redraw();
	void click_func(unsigned func);
	
	void set_anim_timer();
	void func_OpenEx();
	void func_OpenDir();
	void func_ToggleFollow();
	void func_FocusPlaying();
	void func_Refresh();
	void func_NextSource();
	void func_ResetSource();

	bool set_image();
	bool set_image(unsigned fstsource);
	bool set_image(metadb_handle_ptr track);
	bool set_image(image_data current, image_data *next);
	bool set_image(metadb_handle_ptr track, unsigned fstsource);

	void lock(bool l);
	bool is_locked();
};

static uie::window_factory_transparent_single<uie_albumart> foo_uie_albumart;

#include "albumart_config.h"

class NOVTABLE albumart_control_base : public service_base
{
public:
	virtual bool is_controllable()=0;
	virtual bool set_image(metadb_handle_ptr track)=0;
	virtual bool set_image()=0;
	virtual void redraw()=0;
	virtual void lock(bool l)=0;
	virtual bool is_locked()=0;

	FB2K_MAKE_SERVICE_INTERFACE_ENTRYPOINT(albumart_control_base);
};

class albumart_control : public albumart_control_base
{
	virtual bool is_controllable(){
		return cfg_usecontrol && foo_uie_albumart.get_wnd();
	}
	virtual bool set_image(metadb_handle_ptr track)
	{
		if(cfg_usecontrol && foo_uie_albumart.get_wnd())
			return foo_uie_albumart.set_image(track);
		else
			return NULL;
	}
	virtual bool set_image()
	{
		if(cfg_usecontrol && foo_uie_albumart.get_wnd())
			return foo_uie_albumart.set_image();
		else
			return NULL;
	}
	virtual void redraw()
	{
		if(cfg_usecontrol && foo_uie_albumart.get_wnd())
			foo_uie_albumart.redraw();
	}
	virtual void lock(bool l)
	{
		if(cfg_usecontrol && foo_uie_albumart.get_wnd())
			foo_uie_albumart.lock(l);
	}
	virtual bool is_locked()
	{
		if(cfg_usecontrol && foo_uie_albumart.get_wnd())
			return foo_uie_albumart.is_locked();
		else
			return NULL;
	}
};

static service_factory_single_transparent_t<albumart_control> foo_uie_albumart_control;
